
import click
import bisi.interface

from bisi.__version__ import __version__


@click.group(name='bisi')
def cli():
    """bisi cli for building, deploying, and running docker workloads"""
    pass


@cli.command()
def version():
    """Prints the current version of bisi"""
    print(__version__)


@cli.command()
def build():
    """Builds any resources locally"""
    bisi.interface.build()


@cli.command()
def deploy():
    """Deploys resources to cloud providers"""
    bisi.interface.deploy()


@cli.command()
@click.argument('name', type=str)
@click.option('--provider', type=str, default='Local')
@click.argument('arguments', type=str, nargs=-1)
def run(name, provider, arguments):
    """Run NAME with ARGUMENTS """
    bisi.interface.run(name, arguments, provider=provider)


if __name__ == '__main__':
    cli()




import os
import boto3
import base64
import docker
import botocore.exceptions

from typing import Iterable

from bisi.providers.provider import CloudMixin
from bisi.resources import Resource, Dockerfile, Job
from bisi.providers import Provider

ecr_client = boto3.client('ecr')
batch_client = boto3.client('batch')
docker_client = docker.from_env()


class Aws(Provider, CloudMixin):

    def build(self) -> None:
        pass

    def deploy(self) -> None:
        """Deploys the jobs with a BatchJobConfig to AWS Batch and pushes their dockerfiles to ECR"""
        jobs = Resource.get_resources(resource_type=Job)

        for job in jobs:
            if job.batch_config is not None:
                full_repo_name = self.push_to_ecr(dockerfile=job.dockerfile)
                self.create_batch_job_definition(job=job, full_repo_name=full_repo_name)

    @staticmethod
    def ensure_repository_exists(repository_name, **kwargs) -> str:
        """Checks if the repository exists and creates if it doesn't

        Args:
            repository_name: the name of the repository to check/create
            **kwargs: extra args to pass to the create_repository function

        Returns:
            the full repository name
        """
        try:
            repos = ecr_client.describe_repositories(repositoryNames=[repository_name])
            return repos['repositories'][0]['repositoryUri']
        except botocore.exceptions.ClientError:
            repo = ecr_client.create_repository(repositoryName=repository_name, **kwargs)
            return repo['repository']['repositoryUri']

    @staticmethod
    def ensure_ecr_authenticated(server):
        """Logs into ECR so we can push docker images

        Args:
            server: the server to log in to
        """
        auth_creds = ecr_client.get_authorization_token()['authorizationData'][0]
        username, password = base64.b64decode(auth_creds['authorizationToken']).decode('utf-8').split(':')
        ret_val = os.system(f'docker login --username {username} --password {password} '
                            f'{server}')
        if ret_val != 0:
            raise Exception('Failed to login to ECR')

    @staticmethod
    def push_to_ecr(dockerfile: Dockerfile):
        """Pushes the dockerfile to ECR

        Args:
            dockerfile: the dockerfile to push
        """
        full_repo_name = Aws.ensure_repository_exists(dockerfile.ecr_config.repositoryName,
                                                      **dockerfile.ecr_config.kwargs)
        Aws.ensure_ecr_authenticated(full_repo_name.split('/')[0])
        for tag in dockerfile.tags:
            result = docker_client.api.tag(f'{dockerfile.name}:{tag}', full_repo_name, tag=tag, force=True)
            if result is False:
                raise Exception('Failed to tag docker image')

            ret_val = os.system(f'docker push {full_repo_name}:{tag}')
            if ret_val != 0:
                raise Exception('Failed to push docker image')
        return full_repo_name

    @staticmethod
    def get_ecr_image_digest(repo_name, tag):
        paginator = ecr_client.get_paginator('list_images')

        for images in paginator.paginate(repositoryName=repo_name):
            for image in images['imageIds']:
                if image.get('imageTag', None) == tag:
                    return image['imageDigest']
        raise Exception('Could not find ECR image')

    @staticmethod
    def create_batch_job_definition(job: Job, full_repo_name: str):
        """Registers a batch job definition for this job

        Args:
            job: the job to create a definition for
            full_repo_name: the ECR repo name for this job to pull from
        """
        image_digest = Aws.get_ecr_image_digest(job.dockerfile.ecr_config.repositoryName, job.dockerfile.tags[0])
        container_properties = job.batch_config.containerProperties.copy()
        container_properties['image'] = f'{full_repo_name}@{image_digest}'
        batch_client.register_job_definition(
            jobDefinitionName=job.batch_config.jobDefinitionName,
            type=job.batch_config.type,
            containerProperties=container_properties,
            **job.batch_config.kwargs
        )

    def run(self, name: str, arguments: Iterable[str]):
        """Runs the given resources with the given arguments

        Args:
            name: the name of the resource to run
            arguments: a list of arguments to run with
        """
        job = Resource.get_resources(resource_type=Job, name=name)
        if len(job) != 1:
            raise Exception(f'Could not find job with name={name}')
        self.run_job(job[0], arguments)

    @staticmethod
    def run_job(job: Job, arguments: Iterable[str]):
        """Runs the given job with the specified arguments in AWS Batch

        Args:
            job: the job to run
            arguments: a list of arguments to run with
        """
        assert job.batch_config is not None, 'batch_config is required for running a batch job'

        print(f'Running {job}')

        command = [job.entrypoint] + list(arguments) + job.batch_config.extra_args
        if job.entrypoint.endswith('.py'):
            command = ['python'] + command

        environment = job.environment.copy()
        environment.update(job.batch_config.environment)

        batch_client.submit_job(
            jobName=job.name,
            jobQueue=job.batch_config.jobQueue,
            jobDefinition=job.batch_config.jobDefinitionName,
            containerOverrides={
                'command': command,
                'environment': [{'name': key, 'value': value} for key, value in environment.items()]
            }
        )




class ECRConfig:

    def __init__(self, repositoryName: str, **kwargs):
        """Config class for creating an ECR repository
        All params get passed to https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecr.html#ECR.Client.create_repository

        Args:
            repositoryName: the name of the repository to store an image to, bisi will create it if it doesn't exist
            **kwargs: extra arguments to pass to boto3.client('ecr').create_repository
        """
        self.repositoryName = repositoryName
        self.kwargs = kwargs





import os
from pathlib import Path
from typing import Union

from git import Repo, NoSuchPathError, InvalidGitRepositoryError


def get_project_root_dir() -> Path:
    """Gets the current project root dir based on the current directory"""
    repo = get_git_repo()
    if repo is not None:
        return Path(repo.working_tree_dir).absolute()
    else:
        return Path(os.curdir).absolute()


def get_git_repo() -> Union[Repo, None]:
    """Gets the git repo based on current directory, returns none if can not get repo"""
    try:
        return Repo(path=os.curdir, search_parent_directories=True)
    except (NoSuchPathError, InvalidGitRepositoryError):
        return None


def get_git_branch() -> Union[str, None]:
    """Returns the current git branch, returns none if can not get repo"""
    repo = get_git_repo()
    if repo is not None:
        return get_git_repo().active_branch.name


def clean_string(string: str) -> str:
    return string.replace('/', '_')




from typing import Type


class Resource:
    resources = list()

    def __init__(self):
        """A base class for all resources and is also responsible for gathering all the created resources"""
        self.resources.append(self)

    @classmethod
    def get_resources(cls, resource_type: Type = None, name: str = None):
        """Returns the created resources

        Args:
            resource_type: optionally filter by a resource type
            name: optionally filter by a name
        """
        resources = cls.resources
        if resource_type:
            resources = list(filter(lambda x: isinstance(x, resource_type), resources))
        if name:
            resources = list(filter(lambda x: x.name == name, resources))

        return resources

    @classmethod
    def clear_resources(cls):
        """Clears the internal list of resources"""
        cls.resources = list()



import setuptools

with open('src/bisi/__version__.py', 'r') as version_file:
    exec(version_file.read())

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

with open('requirements_dev.txt') as f:
    dev_requirements = f.read().splitlines()[1:]

with open('README.md') as f:
    README = f.read()


setuptools.setup(
    name='bisi',
    version=globals()['__version__'],
    description='A Python based runner for docker images.',
    long_description=README,
    long_description_content_type='text/markdown',
    python_requires='>=3.6.0',
    packages=setuptools.find_packages(where="src", include='bisi.*'),
    package_dir={'': 'src'},
    zip_safe=False,
    install_requires=requirements,
    extras_require={
        'dev': dev_requirements
    },
    entry_points={
        'console_scripts': [
            'bisi = bisi.cli.main:cli'
        ]
    }
)




from typing import Iterable, Union
from pathlib import Path

from bisi.providers import Provider, all_providers
from bisi.resources import collect
from bisi.utils import get_project_root_dir


def run(name: str, arguments: Iterable[str], project_root_dir: Union[str, Path] = None, provider: Union[str, Provider] = 'Local') -> None:
    """Runs a resource with the given arguments and provider

    Args:
        name: the name of the resources to run
        arguments: the arguments to pass to the resource
        project_root_dir: the root directory to build out of
        provider: the provider to run the resource with
    """
    project_root_dir = get_project_root_dir() if project_root_dir is None else Path(project_root_dir)

    if isinstance(provider, str):
        provider = all_providers[provider.lower()](project_root_dir)

    collect.create_resources(project_root_dir)

    provider.run(name, arguments)




from typing import Dict, List


class LocalJobConfig:

    def __init__(self, extra_args: List[str] = None, environment: Dict[str, str] = None, remove: bool = True, **kwargs):
        """Config class for configuring local jobs
        Most arguments get passed to https://docker-py.readthedocs.io/en/stable/containers.html#docker.models.containers.ContainerCollection.run
        image and command are set by bisi so are ignored in this class

        Args:
            extra_args: a list of extra arguments to call the entrypoint with
            environment: a dictionary of environment variables to set for local jobs
            auto_remove: whether to remove the container after it exits
            **kwargs: extra parameters to pass to docker.containers.run
        """
        self.extra_args = extra_args if extra_args else list()
        self.environment = environment if environment else dict()
        self.remove = remove
        self.kwargs = kwargs




from typing import Union
from pathlib import Path

from bisi.providers import all_providers
from bisi.resources import Resource, collect
from bisi.utils import get_project_root_dir


def deploy(project_root_dir: Union[str, Path] = None) -> None:
    """Deploys the resources defined in the project bisi files

    Args:
        project_root_dir: the root directory to build out of

    """
    print('deploying...')
    project_root_dir = get_project_root_dir() if project_root_dir is None else Path(project_root_dir)
    collect.create_resources(project_root_dir)

    for provider_name, provider_class in all_providers.items():
        provider_class(project_root_dir).deploy()
